@@ -53,7 +53,7 @@
 #' from REDCapR API calls. Default `TRUE`.
 #' @param guess_max A positive [base::numeric] value
 #' passed to [readr::read_csv()] that specifies the maximum number of records to
-#' use for guessing column types. Default `.Machine$integer.max`.
+#' use for guessing column types. Default `Inf`.
 #' @param allow_mixed_structure A logical to allow for support of mixed repeating/non-repeating
 #' instruments. Setting to `TRUE` will treat the mixed instrument's non-repeating versions
 #' as repeating instruments with a single instance. Applies to longitudinal projects
@@ -80,7 +80,7 @@ read_redcap <- function(redcap_uri,
                         export_survey_fields = NULL,
                         export_data_access_groups = NULL,
                         suppress_redcapr_messages = TRUE,
-                        guess_max = .Machine$integer.max,
+                        guess_max = Inf,
                         allow_mixed_structure = getOption("redcaptidier.allow.mixed.structure", FALSE)) {
   check_arg_is_character(redcap_uri, len = 1, any.missing = FALSE)
   check_arg_is_character(token, len = 1, any.missing = FALSE)
@@ -260,18 +260,28 @@ read_redcap <- function(redcap_uri,
     db_data <- multi_choice_to_labels(db_data, db_metadata, raw_or_label)
   }
 
-  # Longitudinal Arms Check and Cleaning Application ----
-  # Check if database supplied is longitudinal to determine appropriate function
-  # to use
-  is_longitudinal <- if ("redcap_event_name" %in% names(db_data)) {
-    TRUE
+  # Database structure checks and definitions ----
+  is_longitudinal <- "redcap_event_name" %in% names(db_data)
+  has_repeating_structure <- "redcap_repeat_instance" %in% names(db_data)
+  has_repeating_events <- if (is_longitudinal && has_repeating_structure) {
+    any(
+      is.na(db_data$redcap_repeat_instrument) &
+        !is.na(db_data$redcap_repeat_instance)
+    )
   } else {
     FALSE
   }
 
   if (is_longitudinal) {
+    repeat_event_types <- if (has_repeating_events) {
+      get_repeat_event_types(db_data)
+    } else {
+      NULL
+    }
+
     linked_arms <- link_arms(
-      redcap_uri = redcap_uri, token = token,
+      redcap_uri = redcap_uri,
+      token = token,
       suppress_redcapr_messages = suppress_redcapr_messages
     )
 
@@ -292,7 +302,7 @@ read_redcap <- function(redcap_uri,
   out <- add_metadata(out, db_metadata, redcap_uri, token, suppress_redcapr_messages)
 
   if (is_longitudinal) {
-    out <- add_event_mapping(out, linked_arms)
+    out <- add_event_mapping(out, linked_arms, repeat_event_types)
   }
 
   out <- out %>%
@@ -440,18 +450,27 @@ add_metadata <- function(supertbl, db_metadata, redcap_uri, token, suppress_redc
 #' @param supertbl a supertibble object to supplement with metadata
 #' @param linked_arms the tibble with event mappings created by
 #' \code{link_arms()}
+#' @param repeat_event_types a dataframe output from [get_repeat_event_types()] which
+#' specifies NR, RS, and RT types for events
 #'
 #' @return
 #' The original supertibble with an events \code{redcap_events} list column
 #' containing arms and events associated with each instrument
 #'
 #' @keywords internal
-#'
-add_event_mapping <- function(supertbl, linked_arms) {
-  event_info <- linked_arms %>%
+
+add_event_mapping <- function(supertbl, linked_arms, repeat_event_types) {
+  event_info <- linked_arms
+
+  if (!is.null(repeat_event_types)) {
+    event_info <- event_info %>%
+      left_join(repeat_event_types, by = c("unique_event_name" = "redcap_event_name"))
+  }
+
+  event_info <- event_info %>%
     add_partial_keys(.data$unique_event_name) %>%
     select(
-      redcap_form_name = "form", "redcap_event", "redcap_arm", "arm_name"
+      redcap_form_name = "form", "redcap_event", "event_name", "redcap_arm", "arm_name", any_of("repeat_type")
     ) %>%
     nest(redcap_events = !"redcap_form_name")
 
@@ -498,3 +517,51 @@ calc_metadata_stats <- function(data) {
     form_complete_pct = percent(form_complete_pct, digits = 2, format = "fg")
   )
 }
+
+#' @title
+#' Add identification for repeat event types
+#'
+#' @description
+#' To correctly assign repeat event types a few assumptions must be made:
+#'
+#' - There are only 3 behaviors: nonrepeating, repeat_separately, and repeat_together
+#' - If an event only shows `redcap_repeat_instance` and `redcap_repeat_instrument`
+#' as `NA`, it can be considered a nonrepeat event.
+#' - If an event is always `NA` for `redcap_repeat_instrument` and filled for `redcap_repeat_instance`
+#' it can be assumed to be a repeat_together event
+#' - repeat_separate and nonrepeating event types exhibit the same behavior along the
+#' primary keys of the data. nonrepeating event types can have data display with
+#' `redcap_repeat_instance`values both filled and as `NA`. If this is the case,
+#' it can be assumed the event is a repeating separate event.
+#'
+#' @param data the REDCap data
+#'
+#' @return
+#' A dataframe with unique event names mapped to their corresponding repeat types
+#'
+#' @keywords internal
+
+get_repeat_event_types <- function(data) {
+  out <- data %>%
+    distinct(.data$redcap_event_name, .data$redcap_repeat_instrument, .data$redcap_repeat_instance) %>%
+    mutate(
+      repeat_type = case_when(
+        !is.na(redcap_event_name) & !is.na(redcap_repeat_instrument) & !is.na(redcap_repeat_instance) ~
+          "repeat_separate",
+        !is.na(redcap_event_name) & is.na(redcap_repeat_instrument) & !is.na(redcap_repeat_instance) ~
+          "repeat_together",
+        TRUE ~ "nonrepeating"
+      )
+    ) %>%
+    distinct(.data$redcap_event_name, .data$repeat_type)
+
+  # Check for instances where the same event is labelled as nonrepeating & repeating separate
+  # If this is the case, it must be repeating separate (there is just data that qualifies as both)
+
+  out %>%
+    mutate(
+      is_duplicated = (duplicated(.data$redcap_event_name) | duplicated(.data$redcap_event_name, fromLast = TRUE))
+    ) %>%
+    filter(!.data$is_duplicated | (.data$is_duplicated & .data$repeat_type == "repeat_separate")) %>%
+    select(-.data$is_duplicated)
+}
